/*! \file
  \brief Functions where is it vital that bounds checking is always enabled 
*/

#undef NDEBUG

#include "NMRsim_Process.h"

void handle_bounds(const BadIndex& exc)
{
  std::cerr << "Row/column extraction failed: " << static_cast<const MatrixException&>(exc).what() << '\n';
  error_abort();
}

namespace {
  void scalesw(processing_state& pflags, double scalef)
  {
    if (!(pflags.istimedomain))
      pflags.sw*=scalef;
  }
}

void get_selection(LIST<complex>& dest, const BaseList<complex>& source, const BaseList<size_t>& inds)
{
  try {
    dest=source(inds);
  } catch (BadIndex& exc) {
    handle_bounds(exc);
  }
}

void ProcessExtract::exec(cmatrix& data, BaseList<processing_state> pflags) const
{
  try {
    const size_t oldcols=data.cols();
    const size_t oldrows=data.rows();
    if (oldrows>1) {
      if (row_sel.empty())
	tmp=data(range(),col_sel); //!< don't need to check for empty col_sel
      else {
	if (col_sel.empty())
	  tmp=data(row_sel,range());
	else
	  tmp=data(row_sel,col_sel);
      }
      data.swap(tmp);
      scalesw(pflags.front(),double(data.rows())/oldrows);
    }
    else {
      if (!row_sel.empty())
	error_abort("extract: 1D data set but 2D extract");
      tmpr=data.row()(col_sel);
      data.create(1,tmpr.size(),tmpr.vector());
    }
    scalesw(pflags.back(),double(data.cols())/oldcols);
  } catch (BadIndex& exc) {
    handle_bounds(exc);
  }
}

void check_extract(cmatrix& tmp_set)
{
  cmatrix tmp;
  const LIST<size_t> sel1(parse_unsignedintarray(1));
  try {
    if (parser_isnormal()) {
      const LIST<size_t> col_sel(parse_unsignedintarray(1));
      if (tmp_set.rows()==1)
	error_abort("2D submatrix requested but data is only 1D");
      tmp=tmp_set(sel1,col_sel);
    }
    else
      tmp=tmp_set(range(),sel1);
    tmp_set.swap(tmp);
  } catch (BadIndex& exc) {
    handle_bounds(exc);
  }
}
