// Quadrupole moments of all nuclear isotopes, from P. Pyykko, J. Mol. Phys, 2008 106 1965-1974
// Units of mb, Q/10/fm^2

struct quadrupole_properties {
  const char* label;
  double Q;
};

const quadrupole_properties quadrupole_props_Pyykko08[]={
     {"2H", 2.860},
     {"6Li", -0.808},
     {"7Li", -40.1},
     {"9Be", 52.88},
     {"10B", 84.59},
     {"11B", 40.59},
     {"11C", 33.27},
     {"14N", 20.44},
     {"17O", -25.58},
     {"19F", -94.2},
     {"21Ne", 101.55},
     {"23Na", 104},
     {"25Mg", 199.4},
     {"27Al", 146.6},
     {"33S", -67.8},
     {"35S", 47.1},
     {"35Cl", -81.65},
     {"37Cl", -64.35},
     {"39K", 58.5},
     {"40K", -73},
     {"41K", 71.1},
     {"41Ca", -66.5},
     {"43Ca", -40.8},
     {"45Sc", -220},
     {"47Ti", 302},
     {"49Ti", 247},
     {"50V", 210},
     {"51V", -52},
     {"53Cr", -150},
     {"55Mn", 330},
     {"57Fe", 160},
     {"59Co", 420},
     {"61Ni", 162},
     {"63Cu", -220},
     {"65Cu", -204},
     {"67Zn", 150},
     {"69Ga", 171},
     {"71Ga", 107},
     {"73Ge", -196},
     {"75As", 314},
     {"77Se", 760},
     {"79Br", 313},
     {"81Br", 262},
     {"83Kr", 259},
     {"85Rb", 276},
     {"87Rb", 133.5},
     {"87Sr", 305},
     {"90Y", -125},
     {"91Zr", -176},
     {"93Nb", -320},
     {"95Mo", -22},
     {"97Mo", 255},
     {"99Tc", -129},
     {"99Ru", 79},
     {"101Ru", 457},
     {"105Pd", 660},
     {"113In", 759},
     {"115In", 770},
     {"119Sn", -132},
     {"121Sb", -543},
     {"123Sb", -692},
     {"127I", -696},
     {"131Xe", -114},
     {"133Cs", -3.43},
     {"135Ba", 160},
     {"137Ba", 245},
     {"138La", 450},
     {"139La", 200},
     {"141Pr", -58.9},
     {"143Nd", -630},
     {"145Nd", -330},
     {"147Pm", 740},
     {"147Sm", -259},
     {"149Sm", 75},
     {"151Eu", 903},
     {"153Eu", 2412},
     {"155Gd", 1270},
     {"157Gd", 1350},
     {"159Tb", 1432},
     {"161Dy", 2507},
     {"163Dy", 2648},
     {"165Ho", 3580},
     {"167Er", 3565},
     {"169Tm", -1200}, # Tm missing from Pyykko
     {"173Yb", 2800},
     {"175Lu", 3490},
     {"176Lu", 4970},
     {"177Hf", 3365},
     {"179Hf", 3793},
     {"181Ta", 3170},
     {"185Re", 2180},
     {"187Re", 2070},
     {"189Os", 856},
     {"191Ir", 816},
     {"193Ir", 751},
     {"197Au", 547},
     {"201Hg", 387},
     {"209Pb", -269},
     {"209Bi", -516},
     {"209Rn", 311},
     {"223Fr", 1170},
     {"223Ra", 1210}, # Ra missing from Pyykko
     {"227Ac", 1700},
     {"229Th", 4300},
     {"231Pa", -1720},
     {"233U", 3663},
     {"235U", 4936},
     {"237Np", 3886},
     {"241Pu", 5600},
     {"243Am", 4210},
     {"253Es", 6700},
};

struct default_spin {
  const char* elementname;
  size_t massnumber;
};

const default_spin default_spin_table[] = {
  {"H", 1},
             {"He", 3},
             {"Li", 7},
             {"Be", 9},
             {"B", 11},
             {"C", 13},
             {"O", 17},
             {"F", 19},
             {"Ne", 21},
             {"Na", 23},
             {"Mg", 25},
             {"Al", 27},
             {"Si", 29},
             {"P", 31},
             {"S", 33},
             {"Cl", 35},
             {"K", 39},
             {"Ca", 43},
             {"Sc", 45},
             {"V", 51},
             {"Cr", 53},
             {"Mn", 55},
             {"Fe", 57},
             {"Co", 59},
             {"Ni", 61},
             {"Zn", 67},
             {"Ga", 71},
             {"Ge", 73},
             {"As", 75},
             {"Se", 77},
             {"Br", 81},
             {"Kr", 83},
             {"Rb", 87},
             {"Sr", 87},
             {"Y", 89},
             {"Zr", 91},
             {"Nb", 93},
             {"Mo", 95},
             {"Tc", 99},
             {"Rh", 103},
             {"Pd", 105},
             {"Cd", 113},
             {"In", 115},
             {"Sb", 121},
             {"I", 127},
             {"Sn", 119},
             {"Te", 125},
             {"Cs", 133},
             {"Ba", 137},
             {"La", 139},
             {"Ta", 181},
             {"W", 183},
             {"Re", 187},
             {"Ir", 193},
             {"Pt", 195},
             {"Au", 197},
             {"Tl", 205},
             {"Pb", 207},
             {"Bi", 209}
};


