#include <iostream>
#include <string>
#include <list>

struct SimulateAction {
  virtual void exec(int) =0;
};

struct TranslateAction {
  virtual void exec(const std::string&) =0;
};

template<class ActionPolicy> class ActionBase : public ActionPolicy
{
  virtual void print() =0;
};

template<class ActionImpl> struct ActionEcho : public ActionBase {
  ActionEcho(const char* s) : s_(s), pimpl_(new ActionImpl) {}  
  ~ActionEcho{} { delete pimpl_; }
  void print() { std::cout << "echo " << s_ << '\n'; }
  void exec(int n) { ActionPolicy::exec(n); }
  void exec(const std::string& n) { ActionPolicy::exec(n); }
  std::string s_;
  ActionImpl* pimpl_;
};

void ActionEcho<SimulateAction>::exec(int n)
{
  std::cout << "Now simulating: " << s_ << " with arg " << n << '\n';
}

void ActionEcho<TranslateAction>::exec(const std::string& arg)
{
  std::cout << "Now translating: " << s_ << " with arg " << arg << '\n';
}

int main()
{
  ActionEcho<TranslateAction> actionechot("translate");

  actionechot.print();
  actionechot.exec("X");

  typedef ActionEcho<SimulateAction> ActionEcho_t;
  std::list< ActionEcho_t* > simstack;
  simstack.push_back(new ActionEcho_t("simulate"));
  simstack.front()->print();
  simstack.front()->exec(1);
  return 0;
}
